"""
Main module.
What should it do? Perhaps offer the user the possibility to see what's in the package?
Showing a list of available tools? Like createh5, castorclient, dicomconverttoraw, dicomexplorer, etc.
"""
